@@ -1,6 +1,6 @@
 from functools import reduce, partial
 import operator
-from typing import Any, Dict, Tuple, cast, Optional
+from typing import Any, Dict, Union, Tuple, cast, Optional, Iterator
 
 import copy
 import json
@@ -52,10 +52,10 @@ __all__  = [
 MEMORY_TARGET = int(3.5e9)
 
 def create_blackout_tasks(
-    cloudpath, bounds, 
-    mip=0, shape=(2048, 2048, 64), 
-    value=0, non_aligned_writes=False
-  ):
+  cloudpath:str, bounds:Bbox,
+  mip:int = 0, shape:ShapeType = (2048, 2048, 64), 
+  value:int = 0, non_aligned_writes:bool = False
+):
 
   vol = CloudVolume(cloudpath, mip=mip)
 
@@ -151,7 +151,7 @@ def create_downsampling_tasks(
     sparse=False, bounds=None, chunk_size=None,
     encoding=None, delete_black_uploads=False, 
     background_color=0, dest_path=None, compress=None,
-    factor=None
+    factor=None, bounds_mip=0
   ):
     """
     mip: Download this mip level, writes to mip levels greater than this one.
@@ -205,7 +205,11 @@ def create_downsampling_tasks(
     if not preserve_chunk_size or chunk_size:
       shape = ds_shape(mip + 1, chunk_size, factor)
 
-    bounds = get_bounds(vol, bounds, mip, vol.chunk_size)
+    bounds = get_bounds(
+      vol, bounds, mip,
+      bounds_mip=bounds_mip,
+      chunk_size=vol.chunk_size,
+    )
     
     class DownsampleTaskIterator(FinelyDividedTaskIterator):
       def task(self, shape, offset):
@@ -311,11 +315,19 @@ def create_sharded_image_info(
 
   voxels = prod(dataset_size)
   chunk_voxels = prod(chunk_size)
-  num_chunks = voxels / chunk_voxels
+  num_chunks = Bbox([0,0,0], dataset_size).num_chunks(chunk_size)
 
-  # maximum amount of information in the volume
-  max_bits = math.ceil(math.log2(num_chunks))
-  max_bits = min(max_bits, 64)
+  # maximum amount of information in the morton codes
+  grid_size = np.ceil(Vec(*dataset_size) / Vec(*chunk_size)).astype(np.int64)
+  max_bits = sum([ math.ceil(math.log2(size)) for size in grid_size ])
+  if max_bits > 64:
+    raise ValueError(
+      f"{max_bits}, more than a 64-bit integer, "
+      "would be required to describe the chunk positions "
+      "in this dataset. Try increasing the chunk size or "
+      "increasing dataset bounds."
+      f"Dataset Size: {dataset_size} Chunk Size: {chunk_size}"
+    )
 
   chunks_per_shard = math.ceil(uncompressed_shard_bytesize / (chunk_voxels * byte_width))
   chunks_per_shard = 2 ** int(math.log2(chunks_per_shard))
@@ -323,7 +335,9 @@ def create_sharded_image_info(
   if num_chunks < chunks_per_shard:
     chunks_per_shard = 2 ** int(math.ceil(math.log2(num_chunks)))
 
-  num_shards = num_chunks / chunks_per_shard
+  # approximate, would need to account for rounding effects to be exact
+  # rounding is corrected for via max_bits - pre - mini below.
+  num_shards = num_chunks / chunks_per_shard 
   
   def update_bits():
     shard_bits = int(math.ceil(math.log2(num_shards)))
@@ -358,11 +372,14 @@ def create_sharded_image_info(
       num_shards *= 2
       shard_bits, preshift_bits = update_bits()
 
-  # Preshift bits + shard bits always equals the total information
-  # To have any space for minishard bits, we must steal from
-  # preshift bits, which is okay, because it doesn't affect
-  # which shard bits get read.
+  # preshift_bits + minishard_bits = number of indexable chunks
+  # Since we try to hold the number of indexable chunks fixed, we steal
+  # from preshift_bits to get space for the minishard bits.
+  # We need to make use of the maximum amount of information available
+  # in the morton codes, so if there's any slack from rounding, the
+  # remainder goes into shard bits.
   preshift_bits = preshift_bits - minishard_bits
+  shard_bits = max_bits - preshift_bits - minishard_bits
 
   if preshift_bits < 0:
     raise ValueError(f"Preshift bits cannot be negative. ({shard_bits}, {minishard_bits}, {preshift_bits}), total info: {max_bits} bits")
@@ -391,6 +408,7 @@ def create_image_shard_transfer_tasks(
   chunk_size: Optional[ShapeType] = None,
   encoding: bool = None,
   bounds: Optional[Bbox] = None,
+  bounds_mip : int = 0,
   fill_missing: bool = False,
   translate: ShapeType = (0, 0, 0),
   dest_voxel_offset: Optional[ShapeType] = None,
@@ -448,10 +466,11 @@ def create_image_shard_transfer_tasks(
 
   shape = image_shard_shape_from_spec(spec, dest_vol.scale["size"], chunk_size)
 
-  dest_bounds = get_bounds(dest_vol, bounds, mip, chunk_size)
-
-  if bounds is None:
-    bounds = dest_vol.meta.bounds(mip)
+  bounds = get_bounds(
+    dest_vol, bounds, mip, 
+    bounds_mip=bounds_mip, 
+    chunk_size=chunk_size,
+  )
 
   class ImageShardTransferTaskIterator(FinelyDividedTaskIterator):
     def task(self, shape, offset):
@@ -495,7 +514,7 @@ def create_image_shard_downsample_tasks(
   sparse=False, chunk_size=None,
   encoding=None, memory_target=MEMORY_TARGET,
   agglomerate=False, timestamp=None,
-  factor=(2,2,1), bounds=None
+  factor=(2,2,1), bounds=None, bounds_mip=0
 ):
   """
   Downsamples an existing image layer that may be
@@ -524,7 +543,11 @@ def create_image_shard_downsample_tasks(
   shape = Vec(*shape) * factor
 
   cv.mip = mip
-  bounds = get_bounds(cv, bounds, mip, cv.meta.chunk_size(mip + 1))
+  bounds = get_bounds(
+    cv, bounds, mip, 
+    bounds_mip=bounds_mip, 
+    chunk_size=cv.meta.chunk_size(mip + 1)
+  )
 
   class ImageShardDownsampleTaskIterator(FinelyDividedTaskIterator):
     def task(self, shape, offset):
@@ -614,16 +637,30 @@ def clean_xfer_info(info):
   return info
 
 def create_transfer_tasks(
-    src_layer_path, dest_layer_path, 
-    chunk_size=None, shape=None, 
-    fill_missing=False, translate=None,
-    bounds=None, mip=0, preserve_chunk_size=True,
-    encoding=None, skip_downsamples=False,
-    delete_black_uploads=False, background_color=0,
-    agglomerate=False, timestamp=None, compress='gzip',
-    factor=None, sparse=False, dest_voxel_offset=None,
-    memory_target=MEMORY_TARGET, max_mips=5, clean_info=False
-  ):
+  src_layer_path:str, dest_layer_path:str, 
+  chunk_size:ShapeType = None, 
+  shape:ShapeType = None, 
+  fill_missing:bool = False, 
+  translate:ShapeType = None,
+  bounds:Optional[Bbox] = None, 
+  mip:int = 0, 
+  preserve_chunk_size:bool = True,
+  encoding=None, 
+  skip_downsamples:bool = False,
+  delete_black_uploads:bool = False, 
+  background_color:int = 0,
+  agglomerate:bool = False, 
+  timestamp:Optional[int] = None, 
+  compress:Union[str,bool] = 'gzip',
+  factor:ShapeType = None, 
+  sparse:bool = False, 
+  dest_voxel_offset:ShapeType = None,
+  memory_target:int = MEMORY_TARGET, 
+  max_mips:int = 5, 
+  clean_info:bool = False, 
+  no_src_update:bool = False, 
+  bounds_mip:int = 0,
+) -> Iterator:
   """
   Transfer data to a new data layer. You can use this operation
   to make changes to the dataset representation as well. For 
@@ -665,6 +702,7 @@ def create_transfer_tasks(
     if the memory budget is large enough for more.
   memory_target: given a task size in bytes, pick the task shape that will produce the 
     maximum number of downsamples. Only works for (2,2,1) or (2,2,2).
+  no_src_update: don't update the source's provenance file
   preserve_chunk_size: if true, maintain chunk size of starting mip, else, find the closest
     evenly divisible chunk size to 64,64,64 for this shape and use that. The latter can be
     useful when mip 0 uses huge chunks and you want to simply visualize the upper mips.
@@ -759,7 +797,11 @@ def create_transfer_tasks(
       encoding=encoding
     )
 
-  dest_bounds = get_bounds(dest_vol, bounds, mip, chunk_size)
+  dest_bounds = get_bounds(
+    dest_vol, bounds, mip, 
+    bounds_mip=bounds_mip,
+    chunk_size=chunk_size
+  )
 
   class TransferTaskIterator(FinelyDividedTaskIterator):
     def task(self, shape, offset):  
@@ -815,7 +857,7 @@ def create_transfer_tasks(
       dest_vol.provenance.processing.append(job_details) 
       dest_vol.commit_provenance()
 
-      if src_vol.meta.path.protocol != 'boss':
+      if not no_src_update and src_vol.meta.path.protocol in ('gs', 's3', 'file'):
         src_vol.provenance.processing.append(job_details)
         src_vol.commit_provenance()
 
@@ -825,7 +867,8 @@ def create_contrast_normalization_tasks(
     src_path, dest_path, levels_path=None,
     shape=None, mip=0, clip_fraction=0.01, 
     fill_missing=False, translate=(0,0,0),
-    minval=None, maxval=None, bounds=None
+    minval=None, maxval=None, bounds=None,
+    bounds_mip=0
   ):
 
   srcvol = CloudVolume(src_path, mip=mip)
@@ -848,7 +891,7 @@ def create_contrast_normalization_tasks(
   downsample_scales.create_downsample_scales(dest_path, mip=mip, ds_shape=shape, preserve_chunk_size=True)
   dvol.refresh_info()
 
-  bounds = get_bounds(srcvol, bounds, mip)
+  bounds = get_bounds(srcvol, bounds, mip, bounds_mip=bounds_mip)
 
   class ContrastNormalizationTaskIterator(FinelyDividedTaskIterator):
     def task(self, shape, offset):
@@ -893,7 +936,8 @@ def create_contrast_normalization_tasks(
 
 def create_luminance_levels_tasks(
     layer_path, levels_path=None, coverage_factor=0.01, 
-    shape=None, offset=(0,0,0), mip=0, bounds=None
+    shape=None, offset=(0,0,0), mip=0, 
+    bounds_mip=0, bounds=None
   ):
   """
   Compute per slice luminance level histogram and write them as
@@ -906,24 +950,27 @@ def create_luminance_levels_tasks(
     "coverage_ratio": 0.011, # actual sampled area on this slice normalized by ROI size.
   }
 
-  layer_path: source image to sample from
-  levels_path: which path to write ./levels/ to (default: $layer_path)
-  coverage_factor: what fraction of the image to sample
+  Args:
+    layer_path: source image to sample from
+    levels_path: which path to write ./levels/ to (default: $layer_path)
+    coverage_factor: what fraction of the image to sample
 
-  offset & shape: Allows you to specify an ROI if much of
-    the edges are black. Defaults to entire image.
-  mip: int, which mip to work with, default maximum resolution
+    offset & shape: Allows you to specify an ROI if much of
+      the edges are black. Defaults to entire image.
+    mip (int): which mip to work with, default maximum resolution
+    bounds_mip (int): mip of the input bounds
+    bounds (Bbox-like)
   """
   vol = CloudVolume(layer_path, mip=mip)
 
-  if shape == None:
+  if shape is None:
     shape = Vec(*vol.shape)
     shape.z = 1
 
   offset = Vec(*offset)
   zoffset = offset.clone()
 
-  bounds = get_bounds(vol, bounds, mip)
+  bounds = get_bounds(vol, bounds, mip, bounds_mip=bounds_mip)
   protocol = vol.meta.path.protocol
 
   class LuminanceLevelsTaskIterator(object):
