@@ -1,5 +1,13 @@
-from .skeleton import SkeletonTask, UnshardedSkeletonMergeTask, ShardedSkeletonMergeTask
-from .mesh import MeshTask, MeshManifestTask, GrapheneMeshTask
+from .skeleton import (
+  SkeletonTask, UnshardedSkeletonMergeTask, 
+  ShardedSkeletonMergeTask
+)
+from .mesh import (
+  MeshTask, MeshManifestTask, 
+  GrapheneMeshTask, MeshSpatialIndex,
+  MultiResUnshardedMeshMergeTask,
+  MultiResShardedMeshMergeTask,
+)
 from .image import (
   HyperSquareConsensusTask, #HyperSquareTask,
   DownsampleTask, QuantizeTask, 
